<?php

declare(strict_types=1);

namespace Drupal\views_access_conditions;

use Drupal\Component\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Condition\ConditionAccessResolverTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Condition\ConditionPluginCollection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\Routing\Route;

/**
 * Access check for conditions.
 */
class ConditionsAccessCheck implements AccessInterface {
  use ConditionAccessResolverTrait;

  /**
   * Constructs a new ConditionsAccessCheck object.
   *
   * @param \Drupal\Core\Condition\ConditionManager $conditionManager
   *   The condition plugin manager.
   * @param \Drupal\Core\Plugin\Context\ContextRepositoryInterface $contextRepository
   *   The context repository service.
   * @param \Drupal\Core\Plugin\Context\ContextHandlerInterface $contextHandler
   *   The context handler service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    protected ConditionManager $conditionManager,
    protected ContextRepositoryInterface $contextRepository,
    protected ContextHandlerInterface $contextHandler,
    protected ConfigFactoryInterface $configFactory,
  ) {}

  /**
   * Gets the enabled conditions.
   *
   * @return array
   *   An array of condition IDs.
   */
  protected function getEnabledConditions() {
    $config = $this->configFactory->get('views_access_conditions.settings');
    return $config->get('enabled_conditions') ?: [];
  }

  /**
   * Checks access.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route to check against.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The currently logged in account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(Route $route, AccountInterface $account) {
    $provided_conditions = $route->getRequirement('_conditions');
    $provided_conditions = json_decode($provided_conditions, TRUE);

    if ($provided_conditions === NULL) {
      return AccessResult::neutral();
    }

    // Get enabled conditions.
    $enabled_conditions = $this->getEnabledConditions();

    // If we have specific enabled conditions, filter down to only them.
    if (!empty($enabled_conditions)) {
      $provided_conditions = array_intersect_key(
        $provided_conditions,
        array_flip($enabled_conditions)
      );
    }

    // If there were conditions.
    if (!empty($provided_conditions)) {
      // Ensure each condition config has an 'id' key set to its condition ID.
      $provided_conditions = array_map(function ($config, $condition_id) {
        $config['id'] = $condition_id;
        return $config;
      }, $provided_conditions, array_keys($provided_conditions));

      // Create a condition collection.
      $collection = new ConditionPluginCollection($this->conditionManager, $provided_conditions);
      $collection_array = iterator_to_array($collection);

      $conditions = [];

      // Loop through each condition, verify the contexts are available.
      array_walk($collection_array, function ($condition, $condition_id) use (&$conditions) {
        if ($condition instanceof ContextAwarePluginInterface) {
          // Try to attach any necessary contexts to the condition.
          try {
            $contexts = $this->contextRepository->getRuntimeContexts(array_values($condition->getContextMapping()));
            $this->contextHandler->applyContextMapping($condition, $contexts);
          }
          catch (\Exception $e) {
            // Do nothing.
          }
        }

        $conditions[$condition_id] = $condition;
      });
      // Check the conditions.
      if ($this->resolveConditions($conditions, 'and') === FALSE) {
        // If one or more failed, block access.
        return AccessResult::forbidden();
      }
    }

    // If no conditions are configured, allow access.
    return AccessResult::allowed();
  }

}
