<?php

declare(strict_types=1);

namespace Drupal\views_access_conditions\Plugin\views\access;

use Drupal\conditions_helper\ConditionsEvaluator;
use Drupal\conditions_helper\Form\ConditionsFormBuilder;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\views\Attribute\ViewsAccess;
use Drupal\views\Plugin\views\access\AccessPluginBase;
use Drupal\views_access_conditions\Form\SettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;

/**
 * Access plugin that provides access control based on conditions.
 *
 * @ingroup views_access_plugins
 */
#[ViewsAccess(
  id: "views_access_conditions",
  title: new TranslatableMarkup('Conditions'),
  help: new TranslatableMarkup('Access will be granted if the configured conditions are met.')
)]
class Conditions extends AccessPluginBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  protected $usesOptions = TRUE;

  /**
   * The condition manager.
   *
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * The context repository.
   *
   * @var \Drupal\Core\Context\ContextRepositoryInterface
   */
  protected $contextRepository;

  /**
   * The conditions helper form builder.
   *
   * @var \Drupal\conditions_helper\Form\ConditionsFormBuilder
   */
  protected $conditionsHelperFormBuilder;

  /**
   * The conditions evaluator.
   *
   * @var \Drupal\conditions_helper\ConditionsEvaluator
   */
  protected $conditionsEvaluator;

  /**
   * Constructs a Conditions object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Condition\ConditionManager $condition_manager
   *   The condition manager.
   * @param \Drupal\Core\Context\ContextRepositoryInterface $context_repository
   *   The context repository.
   * @param \Drupal\conditions_helper\Form\ConditionsFormBuilder $conditions_helper_form_builder
   *   The conditions helper form builder.
   * @param \Drupal\conditions_helper\ConditionsEvaluator $conditions_evaluator
   *   The conditions evaluator.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConditionManager $condition_manager, ContextRepositoryInterface $context_repository, ConditionsFormBuilder $conditions_helper_form_builder, ConditionsEvaluator $conditions_evaluator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->conditionManager = $condition_manager;
    $this->contextRepository = $context_repository;
    $this->conditionsHelperFormBuilder = $conditions_helper_form_builder;
    $this->conditionsEvaluator = $conditions_evaluator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.condition'),
      $container->get('context.repository'),
      $container->get('conditions_helper.form_builder'),
      $container->get('conditions_helper.evaluator'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account) {
    return $this->conditionsEvaluator->evaluateConditions($this->options['conditions']);
  }

  /**
   * {@inheritdoc}
   */
  public function alterRouteDefinition(Route $route) {
    if ($this->options['conditions']) {
      $route->setRequirement('_conditions', json_encode($this->options['conditions']));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defineOptions() {
    $options = parent::defineOptions();
    $options['conditions'] = ['default' => []];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['conditions'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    $available_conditions = $this->conditionsHelperFormBuilder->getAvailableConditions(SettingsForm::SETTINGS);
    $available_contexts = $this->contextRepository->getAvailableContexts();
    $stored_values = $this->options['conditions'] ?? [];
    $this->conditionsHelperFormBuilder->buildConditionsForm($form['conditions'], $form_state, $available_conditions, $available_contexts, $stored_values);
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    parent::submitOptionsForm($form, $form_state);

    $cloned_form_state = clone $form_state;
    $cloned_form_state->setValues($form_state->getValue(['access_options']));
    $this->conditionsHelperFormBuilder->submitConditionsForm(
      $form,
      $cloned_form_state,
      ['conditions'],
      ['access_options', 'conditions']
    );
    $values = $cloned_form_state->getValue('conditions');
    $this->options['conditions'] = $values;
  }

  /**
   * {@inheritdoc}
   */
  public function summaryTitle() {
    $summary = [];

    // Loop through each condition.
    array_walk($this->options['conditions'], function ($condition, $id) use (&$summary) {
      // Get the condition definition.
      $definition = $this->conditionManager->getDefinition($id);

      // Create an instance of the condition.
      $instance = $this->conditionManager->createInstance($id, $condition);

      $instance_summary = $instance->summary();
      if (!empty($instance_summary)) {
        // Add the summary to the array.
        $summary[] = $definition['label'] . ': ' . $instance->summary();
      }
    });

    // If there were no conditions, return a default message.
    if (empty($summary)) {
      return $this->t('No conditions configured.');
    }

    // Return the summary.
    return implode(', ', $summary);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();

    // Loop through each condition and add dependencies for each condition type.
    if (!empty($this->options['conditions'])) {
      array_walk($this->options['conditions'], function ($condition, $id) use (&$dependencies) {
        // Get the definition of this condition.
        $definition = $this->conditionManager->getDefinition($id);

        // Add a dependency on the provider for this condition.
        $dependencies['module'][] = $definition['provider'];
      });
    }

    return $dependencies;
  }

}
