<?php

declare(strict_types=1);

namespace Drupal\Tests\views_access_conditions\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\views_access_conditions\Traits\TestConditionTrait;
use Drupal\user\Entity\User;
use Drupal\views\Entity\View;
use Drupal\views\Tests\ViewTestData;

/**
 * Tests the views access conditions functionality.
 *
 * @group views_access_conditions
 */
class ViewsAccessConditionsTest extends KernelTestBase {

  use TestConditionTrait;

  /**
   * The modules to enable for this test.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'text',
    'node',
    'path_alias',
    'views',
    'conditions_helper',
    'views_access_conditions',
    'views_access_conditions_test',
  ];

  /**
   * Views used by this test.
   *
   * @var array
   */
  public static $testViews = ['test_access_conditions'];

  /**
   * The test user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected ?User $user;

  /**
   * The condition manager.
   *
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('node');
    $this->installEntitySchema('path_alias');
    $this->installConfig(['system', 'user', 'field', 'text', 'node', 'path_alias', 'views']);
    $this->installConfig(['views_access_conditions']);

    // Create test views.
    ViewTestData::createTestViews(self::class, ['views_access_conditions_test']);

    // Create a test user.
    $this->user = User::create([
      'name' => 'test_user',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    $this->user->save();

    // Set the current user.
    $this->container->get('current_user')->setAccount($this->user);

    // Get the condition manager.
    $this->conditionManager = $this->container->get('plugin.manager.condition');
  }

  /**
   * Tests that conditions are properly added to the view access form.
   */
  public function testViewAccessFormAlter(): void {
    $view = View::load('test_access_conditions');
    // Configure the view with our test condition.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Verify the conditions were saved correctly.
    $saved_view = View::load('test_access_conditions');
    $saved_display = $saved_view->getDisplay('default');
    $this->assertEquals('views_access_conditions', $saved_display['display_options']['access']['type']);
    $this->assertArrayHasKey('test_condition', $saved_display['display_options']['access']['options']['conditions']);
  }

  /**
   * Tests that conditions properly control view access.
   */
  public function testViewAccessConditions(): void {
    $view = View::load('test_access_conditions');
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    // Configure the view with our test condition.
    $display->setOption('access', [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ]);
    $view->save();

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is allowed when condition is met.
    $this->assertTrue($access_plugin->access($this->user));

    $view = View::load('test_access_conditions');
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    // Change the condition to not be met.
    $display->setOption('access', [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => FALSE,
            'negate' => FALSE,
          ],
        ],
      ],
    ]);
    $view->save();

    $view = View::load('test_access_conditions');
    $display_options = $view->getDisplay('default');
    $this->assertFalse($display_options['display_options']['access']['options']['conditions']['test_condition']['condition_met']);

    // Reset the view executable to ensure we get a fresh instance.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is forbidden when condition is not met.
    $this->assertFalse($access_plugin->access($this->user));
  }

  /**
   * Tests that multiple conditions work together properly.
   */
  public function testMultipleConditions(): void {
    $view = View::load('test_access_conditions');
    // Configure the view with two test conditions.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
          'test_condition_two' => [
            'id' => 'test_condition_two',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Get the view executable.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is allowed when both conditions are met.
    $this->assertTrue($access_plugin->access($this->user));

    // Change one condition to not be met.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
          'test_condition_two' => [
            'id' => 'test_condition_two',
            'condition_met' => FALSE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Get the view executable again.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is forbidden when one condition is not met.
    $this->assertTrue($access_plugin->access($this->user));
  }

  /**
   * Tests the summary title method.
   */
  public function testSummaryTitle(): void {
    $view = View::load('test_access_conditions');
    // Configure the view with our test condition.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Get the view executable.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');
    $access_plugin = $display->getPlugin('access');

    // Assert the summary title is correct.
    $this->assertEquals('Test condition: Test condition is met', $access_plugin->summaryTitle());
  }

  /**
   * Tests the configuration and filtering of available conditions.
   */
  public function testConditionFiltering(): void {
    // Get all available conditions.
    $all_conditions = $this->conditionManager->getDefinitions();

    // Create a test access checker service.
    $access_checker = $this->container->get('views_access_conditions.conditions_access_checker');

    // Test initial state - all conditions available.
    $reflection = new \ReflectionMethod($access_checker, 'getEnabledConditions');
    $reflection->setAccessible(TRUE);
    $enabled_conditions = $reflection->invoke($access_checker);

    // Initially, no conditions should be explicitly enabled.
    $this->assertEmpty($enabled_conditions);

    // Test after configuring specific conditions.
    $test_conditions = array_slice(array_keys($all_conditions), 0, 2);

    // Save the configuration.
    $this->config('views_access_conditions.settings')
      ->set('enabled_conditions', $test_conditions)
      ->save();

    // Get the enabled conditions again.
    $enabled_conditions = $reflection->invoke($access_checker);

    // Verify only the enabled conditions are returned.
    $this->assertEquals($test_conditions, $enabled_conditions);

    // Test the conditions plugin for a view gets filtered conditions.
    $view = View::load('test_access_conditions');
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    // Make sure the access is set to use views_access_conditions.
    $display->setOption('access', [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [],
      ],
    ]);

    // The access plugin itself, when building its form, would use the
    // ConditionsFormBuilder service to get definitions filtered by global
    // settings. We replicate that here to test which conditions are available
    // for selection.
    /** @var \Drupal\conditions_helper\Form\ConditionsFormBuilder $conditionsFormBuilder */
    $conditionsFormBuilder = $this->container->get('conditions_helper.form_builder');
    $available_definitions_for_plugin = $conditionsFormBuilder->getAvailableConditions('views_access_conditions.settings');

    // Check that we only have the enabled conditions available.
    foreach ($test_conditions as $condition_id) {
      $this->assertArrayHasKey($condition_id, $available_definitions_for_plugin);
    }

    // Count total conditions to verify filtering is working.
    $this->assertLessThan(count($all_conditions), count($available_definitions_for_plugin));
    $this->assertEquals(count($test_conditions), count($available_definitions_for_plugin));
  }

}
